@@ -4,7 +4,6 @@ import (
 	"context"
 	"crypto/sha256"
 	"fmt"
-	"path/filepath"
 	"reflect"
 	"strings"
 
@@ -27,14 +26,13 @@ import (
 	"github.com/openshift/library-go/pkg/operator/resource/resourceapply"
 	operatorhelpers "github.com/openshift/library-go/pkg/operator/v1helpers"
 
-	operatorv1client "github.com/open-cluster-management/api/client/operator/clientset/versioned/typed/operator/v1"
-	operatorinformer "github.com/open-cluster-management/api/client/operator/informers/externalversions/operator/v1"
-	operatorlister "github.com/open-cluster-management/api/client/operator/listers/operator/v1"
-	workv1client "github.com/open-cluster-management/api/client/work/clientset/versioned/typed/work/v1"
-	operatorapiv1 "github.com/open-cluster-management/api/operator/v1"
-	"github.com/open-cluster-management/registration-operator/pkg/helpers"
-	"github.com/open-cluster-management/registration-operator/pkg/operators/klusterlet/bindata"
-	"github.com/open-cluster-management/registration-operator/pkg/operators/klusterlet/kube111bindata"
+	operatorv1client "open-cluster-management.io/api/client/operator/clientset/versioned/typed/operator/v1"
+	operatorinformer "open-cluster-management.io/api/client/operator/informers/externalversions/operator/v1"
+	operatorlister "open-cluster-management.io/api/client/operator/listers/operator/v1"
+	workv1client "open-cluster-management.io/api/client/work/clientset/versioned/typed/work/v1"
+	operatorapiv1 "open-cluster-management.io/api/operator/v1"
+	"open-cluster-management.io/registration-operator/manifests"
+	"open-cluster-management.io/registration-operator/pkg/helpers"
 )
 
 const (
@@ -48,30 +46,30 @@ const (
 
 var (
 	crdV1StaticFiles = []string{
-		"manifests/klusterlet/0000_01_work.open-cluster-management.io_appliedmanifestworks.crd.yaml",
-		"manifests/klusterlet/0000_02_clusters.open-cluster-management.io_clusterclaims.crd.yaml",
+		"klusterlet/0000_01_work.open-cluster-management.io_appliedmanifestworks.crd.yaml",
+		"klusterlet/0000_02_clusters.open-cluster-management.io_clusterclaims.crd.yaml",
 	}
 
 	crdV1beta1StaticFiles = []string{
-		"manifests/klusterlet/0001_01_work.open-cluster-management.io_appliedmanifestworks.crd.yaml",
-		"manifests/klusterlet/0001_02_clusters.open-cluster-management.io_clusterclaims.crd.yaml",
+		"klusterlet/0001_01_work.open-cluster-management.io_appliedmanifestworks.crd.yaml",
+		"klusterlet/0001_02_clusters.open-cluster-management.io_clusterclaims.crd.yaml",
 	}
 
 	staticResourceFiles = []string{
-		"manifests/klusterlet/klusterlet-registration-serviceaccount.yaml",
-		"manifests/klusterlet/klusterlet-registration-clusterrole.yaml",
-		"manifests/klusterlet/klusterlet-registration-clusterrolebinding.yaml",
-		"manifests/klusterlet/klusterlet-registration-role.yaml",
-		"manifests/klusterlet/klusterlet-registration-rolebinding.yaml",
-		"manifests/klusterlet/klusterlet-work-serviceaccount.yaml",
-		"manifests/klusterlet/klusterlet-work-clusterrole.yaml",
-		"manifests/klusterlet/klusterlet-work-clusterrolebinding.yaml",
-		"manifests/klusterlet/klusterlet-work-clusterrolebinding-addition.yaml",
+		"klusterlet/klusterlet-registration-serviceaccount.yaml",
+		"klusterlet/klusterlet-registration-clusterrole.yaml",
+		"klusterlet/klusterlet-registration-clusterrolebinding.yaml",
+		"klusterlet/klusterlet-registration-role.yaml",
+		"klusterlet/klusterlet-registration-rolebinding.yaml",
+		"klusterlet/klusterlet-work-serviceaccount.yaml",
+		"klusterlet/klusterlet-work-clusterrole.yaml",
+		"klusterlet/klusterlet-work-clusterrolebinding.yaml",
+		"klusterlet/klusterlet-work-clusterrolebinding-addition.yaml",
 	}
 
 	kube111StaticResourceFiles = []string{
-		"manifests/klusterletkube111/klusterlet-registration-operator-clusterrolebinding.yaml",
-		"manifests/klusterletkube111/klusterlet-work-clusterrolebinding.yaml",
+		"klusterletkube111/klusterlet-registration-operator-clusterrolebinding.yaml",
+		"klusterletkube111/klusterlet-work-clusterrolebinding.yaml",
 	}
 )
 
@@ -185,43 +183,30 @@ func (n *klusterletController) sync(ctx context.Context, controllerContext facto
 	}
 
 	// Start deploy klusterlet components
-	// Check if namespace exists
-	_, err = n.kubeClient.CoreV1().Namespaces().Get(ctx, config.KlusterletNamespace, metav1.GetOptions{})
-	switch {
-	case errors.IsNotFound(err):
-		_, createErr := n.kubeClient.CoreV1().Namespaces().Create(ctx, &corev1.Namespace{
-			ObjectMeta: metav1.ObjectMeta{
-				Name: config.KlusterletNamespace,
-				Annotations: map[string]string{
-					"workload.openshift.io/allowed": "management",
-				},
-			},
-		}, metav1.CreateOptions{})
-		if createErr != nil {
-			_, _, _ = helpers.UpdateKlusterletStatus(ctx, n.klusterletClient, klusterletName, helpers.UpdateKlusterletConditionFn(metav1.Condition{
-				Type: klusterletApplied, Status: metav1.ConditionFalse, Reason: "KlusterletApplyFailed",
-				Message: fmt.Sprintf("Failed to create namespace %q: %v", config.KlusterletNamespace, createErr),
-			}))
-			return createErr
+	// Ensure the existence namespaces for klusterlet and klusterlet addon
+	// Sync pull secret to each namespace
+	namespaces := []string{config.KlusterletNamespace, fmt.Sprintf("%s-addon", config.KlusterletNamespace)}
+	for _, namespace := range namespaces {
+		err := n.ensureNamespace(ctx, klusterlet.Name, namespace)
+		if err != nil {
+			return err
 		}
-	case err != nil:
-		_, _, _ = helpers.UpdateKlusterletStatus(ctx, n.klusterletClient, klusterletName, helpers.UpdateKlusterletConditionFn(metav1.Condition{
-			Type: klusterletApplied, Status: metav1.ConditionFalse, Reason: "KlusterletApplyFailed",
-			Message: fmt.Sprintf("Failed to get namespace %q: %v", config.KlusterletNamespace, err),
-		}))
-		return err
-	}
 
-	// Symc pull secret
-	_, _, err = resourceapply.SyncSecret(
-		n.kubeClient.CoreV1(),
-		controllerContext.Recorder(),
-		n.operatorNamespace,
-		imagePullSecret,
-		config.KlusterletNamespace,
-		imagePullSecret,
-		[]metav1.OwnerReference{},
-	)
+		// Sync pull secret
+		_, _, err = resourceapply.SyncSecret(
+			n.kubeClient.CoreV1(),
+			controllerContext.Recorder(),
+			n.operatorNamespace,
+			imagePullSecret,
+			namespace,
+			imagePullSecret,
+			[]metav1.OwnerReference{},
+		)
+
+		if err != nil {
+			return err
+		}
+	}
 
 	if err != nil {
 		_, _, _ = helpers.UpdateKlusterletStatus(ctx, n.klusterletClient, klusterletName, helpers.UpdateKlusterletConditionFn(metav1.Condition{
@@ -240,7 +225,11 @@ func (n *klusterletController) sync(ctx context.Context, controllerContext facto
 			resourceapply.NewKubeClientHolder(n.kubeClient),
 			controllerContext.Recorder(),
 			func(name string) ([]byte, error) {
-				return assets.MustCreateAssetFromTemplate(name, kube111bindata.MustAsset(filepath.Join("", name)), config).Data, nil
+				template, err := manifests.Klusterlet111ManifestFiles.ReadFile(name)
+				if err != nil {
+					return nil, err
+				}
+				return assets.MustCreateAssetFromTemplate(name, template, config).Data, nil
 			},
 			kube111StaticResourceFiles...,
 		)
@@ -264,7 +253,11 @@ func (n *klusterletController) sync(ctx context.Context, controllerContext facto
 		resourceapply.NewKubeClientHolder(n.kubeClient).WithAPIExtensionsClient(n.apiExtensionClient),
 		controllerContext.Recorder(),
 		func(name string) ([]byte, error) {
-			return assets.MustCreateAssetFromTemplate(name, bindata.MustAsset(filepath.Join("", name)), config).Data, nil
+			template, err := manifests.KlusterletManifestFiles.ReadFile(name)
+			if err != nil {
+				return nil, err
+			}
+			return assets.MustCreateAssetFromTemplate(name, template, config).Data, nil
 		},
 		appliedStaticFiles...,
 	)
@@ -316,11 +309,16 @@ func (n *klusterletController) sync(ctx context.Context, controllerContext facto
 	registrationGeneration, err := helpers.ApplyDeployment(
 		n.kubeClient,
 		klusterlet.Status.Generations,
+		klusterlet.Spec.NodePlacement,
 		func(name string) ([]byte, error) {
-			return assets.MustCreateAssetFromTemplate(name, bindata.MustAsset(filepath.Join("", name)), config).Data, nil
+			template, err := manifests.KlusterletManifestFiles.ReadFile(name)
+			if err != nil {
+				return nil, err
+			}
+			return assets.MustCreateAssetFromTemplate(name, template, config).Data, nil
 		},
 		controllerContext.Recorder(),
-		"manifests/klusterlet/klusterlet-registration-deployment.yaml")
+		"klusterlet/klusterlet-registration-deployment.yaml")
 	if err != nil {
 		_, _, _ = helpers.UpdateKlusterletStatus(ctx, n.klusterletClient, klusterletName, helpers.UpdateKlusterletConditionFn(metav1.Condition{
 			Type: klusterletApplied, Status: metav1.ConditionFalse, Reason: "KlusterletApplyFailed",
@@ -341,11 +339,16 @@ func (n *klusterletController) sync(ctx context.Context, controllerContext facto
 	workGeneration, err := helpers.ApplyDeployment(
 		n.kubeClient,
 		klusterlet.Status.Generations,
+		klusterlet.Spec.NodePlacement,
 		func(name string) ([]byte, error) {
-			return assets.MustCreateAssetFromTemplate(name, bindata.MustAsset(filepath.Join("", name)), config).Data, nil
+			template, err := manifests.KlusterletManifestFiles.ReadFile(name)
+			if err != nil {
+				return nil, err
+			}
+			return assets.MustCreateAssetFromTemplate(name, template, config).Data, nil
 		},
 		controllerContext.Recorder(),
-		"manifests/klusterlet/klusterlet-work-deployment.yaml")
+		"klusterlet/klusterlet-work-deployment.yaml")
 	if err != nil {
 		_, _, _ = helpers.UpdateKlusterletStatus(ctx, n.klusterletClient, klusterletName, helpers.UpdateKlusterletConditionFn(metav1.Condition{
 			Type: klusterletApplied, Status: metav1.ConditionFalse, Reason: "KlusterletApplyFailed",
@@ -369,6 +372,36 @@ func (n *klusterletController) sync(ctx context.Context, controllerContext facto
 	return nil
 }
 
+func (n *klusterletController) ensureNamespace(ctx context.Context, klusterletName, namespace string) error {
+	_, err := n.kubeClient.CoreV1().Namespaces().Get(ctx, namespace, metav1.GetOptions{})
+	switch {
+	case errors.IsNotFound(err):
+		_, createErr := n.kubeClient.CoreV1().Namespaces().Create(ctx, &corev1.Namespace{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: namespace,
+				Annotations: map[string]string{
+					"workload.openshift.io/allowed": "management",
+				},
+			},
+		}, metav1.CreateOptions{})
+		if createErr != nil {
+			_, _, _ = helpers.UpdateKlusterletStatus(ctx, n.klusterletClient, klusterletName, helpers.UpdateKlusterletConditionFn(metav1.Condition{
+				Type: klusterletApplied, Status: metav1.ConditionFalse, Reason: "KlusterletApplyFailed",
+				Message: fmt.Sprintf("Failed to create namespace %q: %v", namespace, createErr),
+			}))
+			return createErr
+		}
+	case err != nil:
+		_, _, _ = helpers.UpdateKlusterletStatus(ctx, n.klusterletClient, klusterletName, helpers.UpdateKlusterletConditionFn(metav1.Condition{
+			Type: klusterletApplied, Status: metav1.ConditionFalse, Reason: "KlusterletApplyFailed",
+			Message: fmt.Sprintf("Failed to get namespace %q: %v", namespace, err),
+		}))
+		return err
+	}
+
+	return nil
+}
+
 func (n *klusterletController) cleanUp(ctx context.Context, controllerContext factory.SyncContext, config klusterletConfig) error {
 	// Remove deployment
 	registrationDeployment := fmt.Sprintf("%s-registration-agent", config.KlusterletName)
@@ -412,7 +445,11 @@ func (n *klusterletController) cleanUp(ctx context.Context, controllerContext fa
 			n.apiExtensionClient,
 			nil,
 			func(name string) ([]byte, error) {
-				return assets.MustCreateAssetFromTemplate(name, bindata.MustAsset(filepath.Join("", name)), config).Data, nil
+				template, err := manifests.KlusterletManifestFiles.ReadFile(name)
+				if err != nil {
+					return nil, err
+				}
+				return assets.MustCreateAssetFromTemplate(name, template, config).Data, nil
 			},
 			file,
 		)
@@ -432,7 +469,11 @@ func (n *klusterletController) cleanUp(ctx context.Context, controllerContext fa
 				nil,
 				nil,
 				func(name string) ([]byte, error) {
-					return assets.MustCreateAssetFromTemplate(name, kube111bindata.MustAsset(filepath.Join("", name)), config).Data, nil
+					template, err := manifests.Klusterlet111ManifestFiles.ReadFile(name)
+					if err != nil {
+						return nil, err
+					}
+					return assets.MustCreateAssetFromTemplate(name, template, config).Data, nil
 				},
 				file,
 			)
@@ -442,10 +483,13 @@ func (n *klusterletController) cleanUp(ctx context.Context, controllerContext fa
 		}
 	}
 
-	// remove the klusterlet namespace
-	err = n.kubeClient.CoreV1().Namespaces().Delete(ctx, config.KlusterletNamespace, metav1.DeleteOptions{})
-	if err != nil && !errors.IsNotFound(err) {
-		return err
+	// remove the klusterlet namespace and klusterlet addon namespace
+	namespaces := []string{config.KlusterletNamespace, fmt.Sprintf("%s-addon", config.KlusterletNamespace)}
+	for _, namespace := range namespaces {
+		err = n.kubeClient.CoreV1().Namespaces().Delete(ctx, namespace, metav1.DeleteOptions{})
+		if err != nil && !errors.IsNotFound(err) {
+			return err
+		}
 	}
 
 	// remove AppliedManifestWorks
@@ -470,7 +514,11 @@ func (n *klusterletController) cleanUp(ctx context.Context, controllerContext fa
 			n.apiExtensionClient,
 			nil,
 			func(name string) ([]byte, error) {
-				return assets.MustCreateAssetFromTemplate(name, bindata.MustAsset(filepath.Join("", name)), config).Data, nil
+				template, err := manifests.KlusterletManifestFiles.ReadFile(name)
+				if err != nil {
+					return nil, err
+				}
+				return assets.MustCreateAssetFromTemplate(name, template, config).Data, nil
 			},
 			file,
 		)
